import {Component, OnInit, ViewChild} from '@angular/core';
import {NavigationLink} from "../../navigation-link-list/navigation-link-list.component";
import * as moment from "moment";
import {ChangeContext, LabelType, Options} from "ngx-slider-v2";
import {
  ChartComponent,
  ApexAxisChartSeries,
  ApexChart,
  ApexXAxis,
  ApexTitleSubtitle, ChartType
} from "ng-apexcharts";
import {timeInterval} from "rxjs";
import {FormControl} from "@angular/forms";
import {HistoryService, TaskgroupPathInfo, TaskgroupService} from "../../../api";

export type ChartOptions = {
  series: ApexAxisChartSeries;
  chart: ApexChart;
  xaxis: ApexXAxis;
  title: ApexTitleSubtitle;
};
@Component({
  selector: 'app-taskgroup-activity',
  templateUrl: './taskgroup-activity.component.html',
  styleUrls: ['./taskgroup-activity.component.css']
})
export class TaskgroupActivityComponent implements OnInit{
  defaultNavigationLinkPath: NavigationLink[] = [
    {
      linkText: 'Dashboard',
      routerLink: ['/']
    },
    {
      linkText: 'Statistics',
      routerLink: []
    },
    {
      linkText: 'Taskgroup Activity',
      routerLink: ['/statistics/taskgroup-activity']
    }
  ];


  selectedChartype: string = "bar";
  availableChartTypes: string[] = ["bar", "line", "area"]
  selectedTaskgroupPath: TaskgroupPathInfo | undefined
  taskgroupPaths: TaskgroupPathInfo[] = []
  sliderControl: FormControl = new FormControl()
  dateRange: Date[] = this.createDateRange();
  selectedDateRange: Date[] = this.dateRange;
  value: number = this.dateRange[0].getTime();
  options: Options = {
    stepsArray: this.dateRange.map((date: Date) => {
      return { value: date.getTime() };
    }),
    translate: (value: number, label: LabelType): string => {
      return new Date(value).toDateString();
    },
    floor: 0,
    ceil: this.dateRange.length,
  };

  @ViewChild("chart") chart?: ChartComponent;
  public chartOptions: Partial<ChartOptions> = this.generateChartOptions()

  constructor(private taskgroupService: TaskgroupService,
              private historyService: HistoryService) {
  }

  ngOnInit() {
    this.taskgroupService.taskgroupsPathGet().subscribe({
      next: resp => {
        this.taskgroupPaths = resp;
      }
    })
  }


  createDateRange(): Date[] {
    const dates: Date[] = [];
    for (let i: number = 1; i <= 31; i++) {

      dates.push(moment().subtract(30-i, 'd').toDate());
    }
    this.sliderControl.setValue([dates[0], dates[dates.length-1]])
    return dates;
  }

  createDateRangeBetween(minValue: moment.Moment, maxValue: moment.Moment) {
    const dates: Date[] = [];
    let currentDate = moment(minValue);

    while(currentDate <= maxValue) {
      dates.push(currentDate.toDate());
      currentDate = currentDate.add(1, 'days');
    }

    return dates;
  }

  onUserChangeStart(changeContext: ChangeContext) {

      //console.log("onUserChangeStart" + new Date(changeContext.value));
  }


  onUserChangeStop(changeContext: ChangeContext) {
      //console.log("onUserChangeStop" + new Date(changeContext.highValue!))
  }

  onUserChange(changeContext: ChangeContext) {
    console.log("min " + moment(changeContext.value).format("YYYY-MM-DD"));
    console.log("max " + moment(changeContext.highValue!).format("YYYY-MM-DD"))
    this.selectedDateRange = this.createDateRangeBetween(moment(changeContext.value), moment(changeContext.highValue!))
    this.chartOptions = this.generateChartOptions()
  }

  generateChartOptions(): Partial<ChartOptions> {
    const series = this.generateSeries();


    return {
      series: series,
        chart: {
          height: 350,
          type: this.selectedChartype as ChartType,
          stacked: true
    },
      title: {
        text: ""
      },
      xaxis: {
        categories: this.selectedDateRange.map(date => date.toDateString())
      }
    };
  }

  generateSeries() : ApexAxisChartSeries {
    const series: ApexAxisChartSeries = []

    if(this.selectedTaskgroupPath != undefined) {
      this.historyService.statisticsTaskgroupActivityTaskgroupIDStartingDateEndingDateIncludeSubTaskgroupsGet(
        this.selectedTaskgroupPath!.rootTasktroup.taskgroupID,
        moment(this.selectedDateRange[0]).format("YYYY-MM-DD"),
        moment(this.selectedDateRange[this.selectedDateRange.length-1]).format("YYYY-MM-DD"),
        false
      ).subscribe({
        next: resp => {
          series.push(
            {
              name: this.selectedTaskgroupPath!.rootTasktroup.taskgroupName,
              data: resp.map(dailyActivityInfo => dailyActivityInfo.activeMinutes)
            }
          );
        }
      })
    }




    this.selectedTaskgroupPath?.directChildren.forEach(taskgroup => {

      this.historyService.statisticsTaskgroupActivityTaskgroupIDStartingDateEndingDateIncludeSubTaskgroupsGet(
        taskgroup.taskgroupID,
        moment(this.selectedDateRange[0]).format("YYYY-MM-DD"),
        moment(this.selectedDateRange[this.selectedDateRange.length-1]).format("YYYY-MM-DD"), true
      ).subscribe({
        next: resp => {
          series.push(
            {
              name: taskgroup.taskgroupName,
              data: resp.map(dailyActivityInfo => dailyActivityInfo.activeMinutes)
            }
          )
        }
      })

    })
    console.log(series);
    return series;

  }

  updateSerieSelection() {
    this.chartOptions = this.generateChartOptions()
  }
}
